﻿using System;
using System.Threading.Tasks;
using StablePoint.Hardware.GhiFezHat;

namespace GadgePiTemplate {
    public sealed partial class MainPage {
        /// <summary>
        /// Initializes FEZ HAT, hiding the async/await stuff.
        /// </summary>
        private void InitializeHat() {
            try {
                Task.Run(async () => {
                    _theHat = await FezHat.CreateAsync();
                    _simulatedLamp = new SimulatedLamp(_theHat);
                }).Wait();
            } catch (Exception) {
                // Swallowing.
            }
        }

        /// <summary>
        /// Uses Hat's PWM11 output to simulate an inert lamp, which takes a couple seconds to go from 0% intensity to 100%.
        /// </summary>
        private class SimulatedLamp {
            private readonly FezHat _localHat;
            private double _targetIntensity;
            private double _actualIntensity;


            public SimulatedLamp(FezHat hat) {
                _localHat = hat;

                Task.Run(async () => {
                    while (true) {
                        await Task.Delay(TimeSpan.FromMilliseconds(33));

                        _localHat.SetPwmDutyCycle(PwmPin.Pwm11, (Math.Pow(10, _actualIntensity) - 1) / 9);

                        if (Math.Abs(_targetIntensity - _actualIntensity) > 0.0001) {
                            var direction = (_actualIntensity - _targetIntensity) < 0 ? 1 : -1;

                            _actualIntensity += direction * 0.01;

                            if (_actualIntensity < 0) _actualIntensity = 0;
                            if (_actualIntensity > 1) _actualIntensity = 1;
                        }
                    }
                });
            }

            /// <summary>
            /// Sets target lamp intensity. It takes some time for the changes to take effect.
            /// </summary>
            /// <param name="intensityToSet">New intensity, from 0.0 to 1.0.</param>
            public void SetIntensity(double intensityToSet) {
                if (intensityToSet < 0) _targetIntensity = 0;
                if (intensityToSet > 1) _targetIntensity = 1;

                _targetIntensity = intensityToSet;
            }
        }
    }
}